/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */

#ifndef _TIMEKEEPER_H_
#define _TIMEKEEPER_H_

//============================================================================
// Name        : time_keeper.h
// Version     : 1.0
// Copyright   :
// Description : スレッド間のデータ共有用．サンプリング周波数を保存して各時刻を変換して返す。
//============================================================================

#include <boost/thread.hpp>
#include <iostream>
#include <Object.h>

namespace beat_tracker {
class TimeKeeper : public FD::Object {
    boost::mutex time_guard_;
    boost::condition_variable state_update_;
public:
    TimeKeeper(size_t sampling_rate, size_t now_samples) {
        sampling_rate_ = sampling_rate;
        now_samples_ = now_samples;
    }
    ;
    ~TimeKeeper() {
    }
    ;
    double get_msec() {
        boost::mutex::scoped_lock lk(time_guard_);
        return now_samples_*1000.0/static_cast<double>(sampling_rate_);
    }
    ;
    size_t get_samples() {
        boost::mutex::scoped_lock lk(time_guard_);
        return now_samples_;
    }
    ;
    size_t get_sampling_rate_() {
        boost::mutex::scoped_lock lk(time_guard_);
        return sampling_rate_;
    }
    ;
    void set_samples(size_t sample) {
        boost::mutex::scoped_lock lk(time_guard_);
        now_samples_=sample;
        state_update_.notify_all();
    }
    ;
    void set_samples_by_diff(int diff) {
        boost::mutex::scoped_lock lk(time_guard_);
        now_samples_+=diff;
        state_update_.notify_all();
    }
    ;
    void WaitForUpdate() {
        boost::mutex::scoped_lock lk(time_guard_);
        state_update_.wait(lk);
    };

    void printOn(std::ostream &out=std::cout) const
    {
        out << "<TimeKeeper " << this->sampling_rate_ << ',' << this->now_samples_ << '>';
    }
private:
    size_t now_samples_;
    size_t sampling_rate_;
};
}

#endif
